!----------------------------------------------------------------------
subroutine atmosphere1(uu,uri,gcov,gcon,detg,x1,nm1,is1,ie1,js1,je1,ks1,ke1)
!----------------------------------------------------------------------
!     floor model for density, pressure, and velocity for vacuum atmosphere  
!
  use pram, only : imax, jmax, kmax, nv, pmin, pmax, dmin, gam
  implicit none
!
  integer :: i, j, k, m, n, nm1, is1, ie1, js1, je1, ks1, ke1, merr

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: gcov(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: gcon(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: detg(is1:ie1,js1:je1,ks1:ke1)
!
  real(8) :: x1(imax)
!
  real(8), allocatable :: gcov1(:,:), gcon1(:,:)
  real(8), allocatable :: util(:), vel(:), beta1(:), bcon(:) 
!
  real(8) :: alpha1, pr, de, demin, prmin, vrmin, dthe, gfl, gflmax, vsq
  real(8) :: rr, rin, epsilon, rde, bsq 
  integer :: isetp1
!
!-allocate variables -!
  allocate(gcov1(0:3,0:3), gcon1(0:3,0:3), stat=merr)
  allocate(util(1:3), vel(1:3), beta1(1:3), bcon(1:3), stat=merr)

!--------------------------------------------------------------------!
!- Parameter -!
  epsilon=1.d-2
  gflmax=10.d0
  isetp1=0
!
!
  do k=ks1+nm1,ke1-nm1
    do j=js1+nm1,je1-nm1
      do i=is1+nm1,ie1-nm1
         
        rr=x1(i)
        rin=1.d0
!
!- copy of primitive variables and conserved variables-! 
        de=uri(1,i,j,k)
        util(1)=uri(2,i,j,k)
        util(2)=uri(3,i,j,k)
        util(3)=uri(4,i,j,k)
        pr=uri(5,i,j,k)
        bcon(1)=uri(7,i,j,k)
        bcon(2)=uri(8,i,j,k)
        bcon(3)=uri(9,i,j,k)
!
!- copy of metric terms -!
        do m=0,3
          do n=0,3
            gcov1(m,n)=gcov(m,n,i,j,k)
            gcon1(m,n)=gcon(m,n,i,j,k)
          enddo
        enddo
!
        alpha1=1./sqrt(-gcon1(0,0))
!
        beta1(1)=alpha1*alpha1*gcon1(0,1)
        beta1(2)=alpha1*alpha1*gcon1(0,2)
        beta1(3)=alpha1*alpha1*gcon1(0,3)
!
        rde=alpha1*uu(1,i,j,k)/detg(i,j,k) !- rde= de*gamma -!
!
!
!- cal of lorentz factor -!        
          call calgfl(util,gfl,gcov1) 
!- cal of contravariant 4-velocity -!
!          call calucon(util,ucon,gfl,gcon1)

!- 4-velocity=> 3-velocity -!
          vel(1)=util(1)/gfl
          vel(2)=util(2)/gfl
          vel(3)=util(3)/gfl
!
!- set atmosphere  value -!         
!        demin=dmin*((rr/rin)**(-3./2.))
!        prmin=pmin*((rr/rin)**(-5./2.))
        demin=dmin
        prmin=pmin
!        vrmin=vel(1)
!        vrmin=0.1d0*vel(1)
!        vrmin=beta1(1)/alpha1
        vrmin=0.d0
        
!- set density limitter   
        dthe=5.0*dmin*(1.d0+epsilon)
!
!- cal bsq
        bsq=bcon(1)*bcon(1)+bcon(2)*bcon(2)+bcon(3)*bcon(3)
!
!- check the density limit -!
!
!        if( de .lt. dthe .and. bsq .eq. 0.d0) then        
        if( de .lt. dthe) then   
!        if( rde .lt. dthe) then
          de=demin
          pr=prmin
          vel(1)=vrmin
          vel(2)=0.d0
          vel(3)=0.d0
          isetp1=1
!
!- new calculation of Lorentz factor -!
          vsq=0.d0
          do m=1,3
            do n=1,3
              vsq=vsq+gcov1(m,n)*vel(m)*vel(n)     
            enddo
          enddo
!
!          if(abs(vsq) .le. 1.d-10) then
!            vsq=1.d-10
!          endif
          if(vsq .ge. 1.d0) then
            vsq=1.-1./(gflmax*gflmax)
          endif
!
!- new set of primitive variables -!
          gfl=1./sqrt(1.-vsq)
          uri(1,i,j,k)=de
          uri(2,i,j,k)=vel(1)*gfl
          uri(3,i,j,k)=vel(2)*gfl
          uri(4,i,j,k)=vel(3)*gfl
          uri(5,i,j,k)=pr
!
        endif
!
      enddo
    enddo
 enddo
!
  if(isetp1 .ne. 0) then
    call caluu3a(uri,uu,gcov,gcon,detg,x1,nm1,is1,ie1,js1,je1,ks1,ke1)
  endif
!
  deallocate(gcov1, gcon1, util, vel, beta1, bcon, stat=merr)
!
  return
end subroutine atmosphere1
!
!----------------------------------------------------------------------
subroutine pminmax1a(uu,uri,gcov,gcon,detg,x1,nm1,is1,ie1,js1,je1,ks1,ke1)
!----------------------------------------------------------------------
  use pram, only : imax, jmax, kmax, nv, pmin, pmax, dmin
  implicit none
!
  integer :: i, j, k, nm1, is1, ie1, js1, je1, ks1, ke1

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: gcov(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: gcon(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: detg(is1:ie1,js1:je1,ks1:ke1)

  real(8) :: x1(imax)
  
  real(8) :: pr, de
  integer :: isetp
!
  isetp=0
!
  do k=ks1+nm1,ke1-nm1
    do j=js1+nm1,je1-nm1
      do i=is1+nm1,ie1-nm1

        isetp=0
         
        pr=uri(5,i,j,k)
        de=uri(1,i,j,k)
         
        if( pr.lt.pmin .or. pr.gt.pmax .or. de.lt.dmin) then
!
          if( pr.lt.pmin ) then
            pr=pmin
            isetp=1
          endif
!
          if( pr.gt.pmax ) then
            pr=pmax
            isetp=2
          endif

          if( de.lt.dmin ) then
            de=dmin
            isetp=3
          endif
!
          uri(5,i,j,k)=pr
          uri(1,i,j,k)=de
!
        endif

      enddo
    enddo
  enddo  
!     
  if(isetp .ne. 0) then
    call caluu3a(uri,uu,gcov,gcon,detg,x1,nm1,is1,ie1,js1,je1,ks1,ke1)
  endif
!
  if( isetp .eq. 1 ) then
    write(6,*) 'Correction: pmin in pminmax'
    write(6,*) 'pmin =', pmin
  elseif( isetp .eq. 2) then
    write(6,*) 'Correction: pmax in pminmax'
    write(6,*) 'pmax =', pmax
  elseif( isetp .eq. 3) then
    write(6,*) 'Correction: dmin in pminmax'
    write(6,*) 'dmin =', dmin
  endif
!
  return
end subroutine pminmax1a
!
!----------------------------------------------------------------------
subroutine pminmax2a(uu,uri,gcov,gcon,detg,x1,nm1,is1,ie1,js1,je1,ks1,ke1)
!----------------------------------------------------------------------
!     floor model for density and pressure                       
!
  use pram, only : imax, jmax, kmax, nv, pmin, pmax, dmin, gam
  implicit none
!
  integer :: i, j, k, nm1, is1, ie1, js1, je1, ks1, ke1, merr

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: gcov(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: gcon(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: detg(is1:ie1,js1:je1,ks1:ke1)
!
  real(8) :: x1(imax)
!
  real(8) :: pr, de, demin, uumin, prmin, rr, rin
!  integer, allocatable :: isetp(:,:,:)
  integer :: isetp1

!  allocate(isetp(is1:ie1,js1:je1,ks1:ke1), stat=merr)
!
!
!  do k=ks1+nm1,ke1-nm1
!    do j=js1+nm1,je1-nm1
!      do i=is1+nm1,ie1-nm1  
!        isetp(i,j,k)=0
!      enddo
!    enddo
!  enddo
  isetp1=0 
!
  do k=ks1+nm1,ke1-nm1
    do j=js1+nm1,je1-nm1
      do i=is1+nm1,ie1-nm1
         
        rr=x1(i)
        rin=1.d0

        pr=uri(5,i,j,k)
        de=uri(1,i,j,k)
         
!        demin=dmin*((rr/rin)**(-3./2.))
!        prmin=pmin*((rr/rin)**(-5./2.))
        demin=dmin
        prmin=pmin
!
        if( pr.lt.prmin .or. pr.gt.pmax .or. de.lt.demin) then
!
          if( pr.lt.prmin ) then
            pr=prmin
!            isetp(i,j,k)=1
            isetp1=1
         endif
!
          if( pr.gt.pmax ) then
            pr=pmax
!            isetp(i,j,k)=2
            isetp1=1
          endif

          if( de.lt.demin ) then          
            de=demin
!            isetp(i,j,k)=3
            isetp1=1
          endif          
!
          uri(5,i,j,k)=pr
          uri(1,i,j,k)=de
!
       endif
!
      enddo
    enddo
 enddo
!
  if(isetp1 .ne. 0) then
    call caluu3a(uri,uu,gcov,gcon,detg,x1,nm1,is1,ie1,js1,je1,ks1,ke1)
  endif
!
!  do k=ks1+nm1,ke1-nm1
!    do j=js1+nm1,je1-nm1
!      do i=is1+nm1,ie1-nm1
!        if( isetp(i,j,k) .eq. 1) then
!          write(6,*) 'Correction: pressure min'
!          write(6,*) 'pmin =', pmin, i, j, k            
!        elseif( isetp(i,j,k) .eq. 2) then
!          write(6,*) 'Correction: pressure max'
!          write(6,*) 'pmax =', pmax, i, j, k
!        elseif( isetp(i,j,k) .eq. 3) then
!          write(6,*) 'Correction: density min'
!          write(6,*) 'dmin =', dmin, i, j, k
!        endif
!      enddo
!    enddo
!  enddo  
!
!  deallocate(isetp, stat=merr)
!
  return
end subroutine pminmax2a
!
!***********************************************************************
!                   ARTIFICIAL DAMP  
!***********************************************************************
!-----------------------------------------------------------------------
subroutine damp4(uu,uri,uri0,gcov,gcon,detg,x1,x3,is1,ie1,js1,je1,ks1,ke1)
!-----------------------------------------------------------------------
  use pram, only : imax, jmax, kmax, nv, metric, adamp, rdamp, &
                   xmin, xmax
  implicit none
  
  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri0(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: gcov(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: gcon(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: detg(is1:ie1,js1:je1,ks1:ke1)

  real(8) :: x1(imax), x3(kmax)
  real(8) :: rr, adt, de, util1, util2, util3, pr, b1, b2, b3
!
  real(8) :: drdamp, dx1a

  drdamp=0.4d0
  dx1a=(xmax-xmin)/float(imax)

  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1
        if(rdamp .gt. 0.d0) then
          if(metric.eq.3 .or. metric.eq.103 .or. metric.eq.203) then
            rr=x1(i)
          else
            rr=sqrt(x1(i)**2+x3(k)**2)
          endif
            adt=adamp*0.5*(1.0+tanh((rdamp-rr)*(0.3/dx1a)))
!            adt=adamp*0.5*(1.0+tanh((rdamp-rr)/drdamp))

            de= uri(1,i,j,k)-adt*(uri(1,i,j,k)-uri0(1,i,j,k))
            util1= uri(2,i,j,k)-adt*(uri(2,i,j,k)-uri0(2,i,j,k))
            util2= uri(3,i,j,k)-adt*(uri(3,i,j,k)-uri0(3,i,j,k))
            util3= uri(4,i,j,k)-adt*(uri(4,i,j,k)-uri0(4,i,j,k))
            pr= uri(5,i,j,k)-adt*(uri(5,i,j,k)-uri0(5,i,j,k))
!            b1= uri(7,i,j,k)-adt*(uri(7,i,j,k)-uri0(7,i,j,k))
!            b2= uri(8,i,j,k)-adt*(uri(8,i,j,k)-uri0(8,i,j,k))
!            b3= uri(9,i,j,k)-adt*(uri(9,i,j,k)-uri0(9,i,j,k))
            b1=uri(7,i,j,k)
            b2=uri(8,i,j,k)
            b3=uri(9,i,j,k)
          
            uri(1,i,j,k)=de
            uri(2,i,j,k)=util1
            uri(3,i,j,k)=util2
            uri(4,i,j,k)=util3
            uri(5,i,j,k)=pr
            uri(7,i,j,k)=b1
            uri(8,i,j,k)=b2
            uri(9,i,j,k)=b3
          
        endif
      enddo
    enddo
  enddo
      
  if(rdamp .gt. 0.d0) then
    call caluu2a(uri,uu,gcov,gcon,detg,is1,ie1,js1,je1,ks1,ke1)
  endif
!
  return
end subroutine damp4
!
!-----------------------------------------------------------------------
subroutine damp4a(uu,uri,uri0,gcov,gcon,detg,x1,x3,is1,ie1,js1,je1,ks1,ke1)
!-----------------------------------------------------------------------
!    set damping zone in z-direction (for jet propagation simulation)
!
  use pram, only : imax, jmax, kmax, nv, metric, adamp, pi
  implicit none
  
  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri(nv,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uri0(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: gcov(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: gcon(0:3,0:3,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: alpha(is1:ie1,js1:je1,ks1:ke1)
  real(8) :: beta(0:3,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: detg(is1:ie1,js1:je1,ks1:ke1)

  real(8) :: x1(imax), x3(kmax)
  real(8) :: zz, zz1, zz2, adt, de, util1, util2, util3, pr, b1, b2, b3, tmp1
!
  zz1=-0.95d0
  zz2=-0.025d0

  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1
        if(adamp .gt. 0.d0) then
          if(metric.eq.3 .or. metric.eq.103 .or. metric.eq.203) then
            zz=x1(i)*cos(x3(k))
          else
            zz=x3(k)
          endif
            tmp1=(zz-zz1)/(zz2-zz1)
            adt=adamp*0.5*(cos(tmp1*pi)+1.0)

            de= uri(1,i,j,k)-adt*(uri(1,i,j,k)-uri0(1,i,j,k))
            util1= uri(2,i,j,k)-adt*(uri(2,i,j,k)-uri0(2,i,j,k))
            util2= uri(3,i,j,k)-adt*(uri(3,i,j,k)-uri0(3,i,j,k))
            util3= uri(4,i,j,k)-adt*(uri(4,i,j,k)-uri0(4,i,j,k))
            pr= uri(5,i,j,k)-adt*(uri(5,i,j,k)-uri0(5,i,j,k))
!            b1= uri(7,i,j,k)-adt*(uri(7,i,j,k)-uri0(7,i,j,k))
!            b2= uri(8,i,j,k)-adt*(uri(8,i,j,k)-uri0(8,i,j,k))
!            b3= uri(9,i,j,k)-adt*(uri(9,i,j,k)-uri0(9,i,j,k))
            b1=uri(7,i,j,k)
            b2=uri(8,i,j,k)
            b3=uri(9,i,j,k)
          
            uri(1,i,j,k)=de
            uri(2,i,j,k)=util1
            uri(3,i,j,k)=util2
            uri(4,i,j,k)=util3
            uri(5,i,j,k)=pr
            uri(7,i,j,k)=b1
            uri(8,i,j,k)=b2
            uri(9,i,j,k)=b3
          
        endif
      enddo
    enddo
  enddo
      
  if(adamp .gt. 0.d0) then      
    call caluu2a(uri,uu,gcov,gcon,detg,is1,ie1,js1,je1,ks1,ke1)
  endif
  
  return
end subroutine damp4a
!
